@@ -25,9 +25,13 @@
 
 # Tiny AST for the MISS_HIT configuration files
 
+import os
+from glob import glob
+
 from abc import ABCMeta, abstractmethod
 
-from miss_hit_core.errors import Message_Handler, ICE
+from miss_hit_core.m_ast import MATLAB_Token
+from miss_hit_core.errors import Location, Message_Handler, ICE
 from miss_hit_core.config import (Config,
                                   Boolean_Style_Configuration,
                                   Integer_Style_Configuration,
@@ -265,3 +269,162 @@ class Octave_Mode(Config_Item):
         assert isinstance(config, Config)
 
         config.octave = self.enabled
+
+
+class Path_List:
+    def __init__(self, root):
+        assert isinstance(root, str)
+        self.root = root
+
+        self.s_paths = set()
+        self.l_paths = []
+
+    def add_path(self, mh, t_path):
+        assert isinstance(t_path, MATLAB_Token)
+
+        names = [os.path.relpath(path, self.root)
+                 for path in glob(os.path.join(self.root,
+                                               t_path.value))]
+
+        if names:
+            for name in sorted(names):
+                if name in self.s_paths:
+                    mh.config_error(t_path.location,
+                                    "duplicate/overlapping path %s" % name)
+                elif not os.path.isdir(os.path.join(self.root, name)):
+                    mh.config_error(t_path.location,
+                                    "%s is not a directory" % name)
+
+                self.s_paths.add(name)
+                self.l_paths.append(name)
+        else:
+            # Python docs say "possibly empty list", not sure how
+            # though. If you don't match, it seems to return the
+            # wild-card verbatim.
+            mh.config_error(t_path.location,
+                            "does not exist")
+
+    def get_path(self):
+        if self.l_paths:
+            # Our path list is just what is specified, in order.
+            return [os.path.join(self.root, path)
+                    for path in self.l_paths]
+
+        else:
+            # If we do not have any paths specified, so the only path
+            # is the item's root.
+            return [self.root]
+
+
+class Project_Directive(Config_Item):
+    def __init__(self, location, directory, name):
+        super().__init__()
+
+        assert isinstance(location, Location)
+        self.location = location
+        # Location if we need to raise errors
+
+        assert isinstance(directory, str)
+        assert os.path.isdir(directory)
+        self.directory = directory
+        # The directory this item is rooted in
+
+        assert isinstance(name, str)
+        self.name = name
+        # Name (also what will be used in the symbol table)
+
+    def evaluate(self, mh, config):
+        raise ICE("logic error - called evaluate() for project directive")
+
+    @abstractmethod
+    def validate(self, mh, symbol_table):
+        pass
+
+    @abstractmethod
+    def get_path(self):
+        # Get the actual PATH for this library. This decides the
+        # search order for user-defined functions and classes.
+        pass
+
+
+class Library_Declaration(Project_Directive):
+    def __init__(self, location, directory, name=None):
+        if name is None:
+            name = os.path.basename(os.path.abspath(directory))
+        super().__init__(location, directory, name)
+
+        self.path_list = Path_List(directory)
+        self.is_global = False
+
+    def __str__(self):
+        return "Library(%s)" % self.name
+
+    def add_path(self, mh, t_path):
+        self.path_list.add_path(mh, t_path)
+
+    def dump(self):
+        print("  Library Declaration (%s)" % self.name)
+        for t_path in self.path_list.l_paths:
+            print("    Path: %s" % t_path.value)
+
+    def validate(self, mh, symbol_table):
+        pass
+
+    def get_path(self):
+        return self.path_list.get_path()
+
+    def set_global(self):
+        self.is_global = True
+
+
+class Entrypoint_Declaration(Project_Directive):
+    def __init__(self, location, directory, name):
+        super().__init__(location, directory, name)
+
+        self.path_list = Path_List(directory)
+
+        self.l_libraries = []
+        self.s_libraries = set()
+        # Libraries that this entry-point depends on. Initially this
+        # is a list of tokens, but after validation is replaced with a
+        # list of Library_Declaration nodes.
+
+    def add_path(self, mh, t_path):
+        self.path_list.add_path(mh, t_path)
+
+    def add_lib_dependency(self, mh, t_lib):
+        assert isinstance(t_lib, MATLAB_Token)
+
+        if t_lib.value in self.s_libraries:
+            mh.config_error(t_lib.location,
+                            "duplicate library")
+
+        self.s_libraries.add(t_lib.value)
+        self.l_libraries.append(t_lib)
+
+    def dump(self):
+        print("  Entrypoint Declaration (%s)" % self.name)
+        for t_path in self.path_list.l_paths:
+            print("    Path: %s" % t_path.value)
+        for t_lib in self.l_libraries:
+            print("    Requires library: %s" % t_lib)
+
+    def validate(self, mh, symbol_table):
+        resolved_library_list = []
+        for t_lib in self.l_libraries:
+            if t_lib.value not in symbol_table:
+                mh.config_error(t_lib.location,
+                                "cannot find library")
+            else:
+                resolved_library_list.append(symbol_table[t_lib.value])
+        self.l_libraries = resolved_library_list
+
+    def get_path(self):
+        # We start with the directory this entry point is in.
+        path = self.path_list.get_path()
+
+        # And now add, in order, the libraries' paths
+        for n_lib in self.l_libraries:
+            path += n_lib.get_path()
+
+        return path
